#Copyright (c) 2017 konstvest

#This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.

#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.

#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.

bl_info = {
    "name": "EI figer",
    "author": "konstvest",
    "version": (1, 1),
    "blender": (2, 78, 0),
    "location": "",
    "description": "Addon for import-export model Evil Islands <-> Blender (without morphing and animations)",
    "wiki_url": "",
    "tracker_url": "konstvest@gmail.com",
    "category": "Import-Export"}

import bpy
import os
from math import sqrt
import copy
from struct import pack,unpack
import bmesh

class EImodelPanel(bpy.types.Panel):
    bl_label = "EI import-export"
    bl_space_type = 'VIEW_3D'
    bl_region_type = 'TOOLS'
    bl_category = 'Tools'
 
    def draw_header(self, context):
        layout = self.layout
        layout.label(text="", icon="PACKAGE")
 
    def draw(self, context):
        layout = self.layout
 
        layout.operator("object.eimodelimport", text="Import")
        layout.label (text="============")
        layout.operator("object.eifigexport", text="Export")
        row = layout.row()
        split = row.split(percentage=0.5)
        col_left = split.column()
        col_right = split.column()
        col_left.prop (context.scene, "group")
        col_right.prop(context.scene, "t_number")
        layout.prop(context.scene, "scale")
 
class OBJECT_OT_EIfigExport(bpy.types.Operator):
    bl_label = "EI figure export Operator"
    bl_idname = "object.eifigexport"
    bl_description = "Exporting figure file for Evil Islands"
    filepath = bpy.props.StringProperty(subtype='FILE_PATH')
    
    def execute(self, context):
        self.report({'INFO'}, "executing export")
        path_file=self.filepath
        
        def triangulate_object(obj):
            me = obj.data
            bm = bmesh.new()
            bm.from_mesh(me)
            bmesh.ops.triangulate(bm, faces=bm.faces[:], quad_method=0, ngon_method=0)
            bm.to_mesh(me)
            bm.free()
        
        def meshclean():
            for mesh in bpy.data.meshes:
                if mesh.users==0:
                    bpy.data.meshes.remove(mesh)        
        
        def export_lnk():
            '''
            export *.lnk. head of lnk is the last of objects (it's need only for v 1.0)
            '''
            link_count=0
            for lobj in bpy.data.objects:
                if lobj.type == 'MESH':
                    lbase_mesh=lobj.name
                    link_count+=1
            fl=open (path_file, 'wb')
            flr=fl.write
            flr(pack('i', link_count))
            hd=len(lbase_mesh)
            lbs_msh=lbase_mesh.encode()
            hd_frmt=str(hd+1)+'s'
            #lnk head
            flr(pack('i', hd+1))
            flr(pack(hd_frmt, lbs_msh))
            flr(pack('i', 0))
            #lnk body
            for lobj in bpy.data.objects:
                if lobj.type == 'MESH':
                    if lbase_mesh!=lobj.name:
                        lname_lenght=len(lobj.name)+1
                        flr(pack('i', lname_lenght))
                        flr(pack(str(lname_lenght)+'s', lobj.name.encode()))
                        flr(pack('i', hd+1))
                        flr(pack(hd_frmt, lbs_msh))
            fl.close()
            print ('lnk exported')
            
            
        def export_bon():
            '''
            set pivot of objects in (0.0, 0.0, 0.0) and save file with 0x96 (it's need only for v 1.0)
            '''
            for bobj in bpy.data.objects:
                if bobj.type=='MESH':
                    bobj.select=False
            for bobj in bpy.data.objects:
                if bobj.type == 'MESH':
                    bobj.select=True
                    bpy.context.scene.cursor_location=(0.0, 0.0, 0.0)
                    bpy.ops.object.origin_set(type='ORIGIN_CURSOR')
                    fb=open (os.path.dirname(path_file)+'\\'+bobj.name+'.bon', 'wb')
                    nyl='\x00'
                    fb.write (pack('96s', nyl.encode()))
                    fb.close()
                    bobj.select=False
            print ('bon(f) export finished')

        def export_fig():
            print ('executing export fig')
            class ei_mesh():
                pass
            rou=6
            for mesh in bpy.data.meshes:
                count_vert=0
                count_norm=0
                v_restore=0
                ind_count=0
                v_c_count=0
                m_c_count=0
                ind_count=0
                
                act_m=ei_mesh()
                act_m.header=[0 for i in range (9)]
                act_m.center=[[], [], []]
                act_m.min=[[], [], []]
                act_m.max=[[], [], []]
                act_m.radius=[]
                act_m.m_verts=[[], [], []]
                act_m.normals=[]
                act_m.t_coords=[]
                act_m.indicies=[]
                act_m.v_c=[]
                
                #VERTICES
                for mvert in mesh.vertices:
                    for mv in range (3):
                        act_m.m_verts[mv].append(round(mvert.co[mv], rou))
                    count_vert+=1
                    #MIN & MAX
                    if mvert.index == 0:
                            min = copy.copy(mvert.co)
                            max = copy.copy(mvert.co)
                    mi=0
                    while mi < 3:
                        if max[mi] < mvert.co[mi]:
                            max[mi]=mvert.co[mi]
                        if min[mi] > mvert.co[mi]:
                            min[mi]=mvert.co[mi]
                        mi+=1
                #align vertices
                if count_vert%4 !=0:
                    v_restore=4-count_vert%4
                for mvert_restore in range (v_restore):
                    for mvr in range (3):
                        act_m.m_verts[mvr].append(0.0)
                    count_vert+=1
                if count_vert%4 ==0:
                    print ('verts now: '+str(count_vert)+' added: '+str(v_restore))
                act_m.header[0]=int(count_vert/4)
                #MIN
                for m0 in range (3):
                    act_m.min[m0].append(round(min[m0], rou))
                #MAX
                for m1 in range (3):
                    act_m.max[m1].append(round(max[m1], rou))
                #RADIUS
                rad = sqrt((max[0]-min[0])**2+(max[1]-min[1])**2+(max[2]-min[2])**2)/2
                act_m.radius.append(round(rad, rou))
                #REAL CENTER
                for mcntr in range (3):
                    tmp=(act_m.min[mcntr][0]+act_m.max[mcntr][0])/2
                    act_m.center[mcntr].append(tmp)
                #NORMALS
                for v_nrmls in mesh.vertices:
                    act_m.normals.append( [round(v_nrmls.normal[0], rou), round(v_nrmls.normal[1], rou), round(v_nrmls.normal[2], rou), 1.0] )
                    count_norm+=1
                print (count_norm)
                print (len(act_m.normals))
                if len(act_m.normals)%4 != 0:
                    for v_norn_restore in range (4-len(act_m.normals)%4):
                        print ('len: '+str(len(act_m.normals))+'\tcount: '+str(count_norm))
                        act_m.normals.append(copy.copy(act_m.normals[count_norm-1]))
                        count_norm+=1
                print ('normals: '+str(len(act_m.normals)/4))
                act_m.header[1]=int(len(act_m.normals)/4)
                ind_ar=[]
                for mpoly in mesh.polygons:
                    #INDICES PREPARE
                    for poly_vrt in mpoly.vertices:
                        ind_ar.append(poly_vrt)
                        ind_count+=1
                #UV COORDS PREPARE
                uv_ar=[]   #array with all t_coords
                uv_temp=[]
                uv_list=[]  #array with unique t_coords
                uv_ar=[]
                new_uv_ind=[]
                uv_counter=0
                for uv_act in mesh.uv_layers.active.data:   #get only active layer with uv_cords
                    uv_temp=[round (uv_act.uv[0], rou), round (uv_act.uv[1], rou)]
                    uv_ar.append(copy.copy(uv_temp))
                    if uv_temp not in act_m.t_coords:
                        act_m.t_coords.append(copy.copy(uv_temp))
                    uv_counter+=1
                act_m.header[2]=len(act_m.t_coords)        
                act_m.header[3]=ind_count
                for uv_ind1 in range (len(uv_ar)):      #get indicies of new t_coords array
                    for uv_ind2 in range (len(act_m.t_coords)):
                        if uv_ar[uv_ind1]==act_m.t_coords[uv_ind2]:
                            new_uv_ind.append(uv_ind2)
                #VERTEX COMPONENTS
                v_c_ar=[]
                for n_i in range (len(ind_ar)):
                    uv_temp=[ind_ar[n_i], new_uv_ind[n_i]]
                    if uv_temp not in act_m.v_c:
                        act_m.v_c.append (copy.copy(uv_temp))
                swap_pts=0
                for bub in range (len(act_m.v_c)):
                    for buble in range (len(act_m.v_c)-1):
                        if act_m.v_c[buble][0] > act_m.v_c[buble+1][0]:
                            swap_pts=copy.copy(act_m.v_c[buble+1])
                            act_m.v_c[buble+1]=copy.copy(act_m.v_c[buble])
                            act_m.v_c[buble]=copy.copy(swap_pts)
                        elif act_m.v_c[buble][0] == act_m.v_c[buble+1][0]:
                            if act_m.v_c[buble][1] > act_m.v_c[buble+1][1]:
                                swap_pts=copy.copy(act_m.v_c[buble+1])
                                act_m.v_c[buble+1]=copy.copy(act_m.v_c[buble])
                                act_m.v_c[buble]=copy.copy(swap_pts)
                act_m.header[4]=len(act_m.v_c)
                #INDICIES
                for mix in range (len(ind_ar)):
                    for mix1 in range (len(act_m.v_c)):
                        if (ind_ar[mix]==act_m.v_c[mix1][0]) & (new_uv_ind[mix]==act_m.v_c[mix1][1]):
                            act_m.indicies.append(mix1)
                            break
                if count_vert!=0:
                    act_m.header[5]=count_vert
                #group of object
                act_m.header[7]=context.scene.group
                #texture number
                act_m.header[8]=context.scene.t_number

                #SAVE IN FILE(s)
                fig_file = os.path.dirname(path_file)+'\\'+mesh.name+".fig"
                fg=open (fig_file, 'wb')
                fgr = fg.write
                fgr (b'FIG8')
                #print ('\theader')
                for h in range (9):
                    fgr (pack ('i', act_m.header[h]))
                #print ('\tcenter')
                for tmp in range (8):  #kostil' dlya bezmorphinga
                    for xyz in range (3):
                        if tmp > 3:
                            fgr (pack ('f', act_m.center[xyz][0]*context.scene.scale))
                        else:
                            fgr (pack ('f', act_m.center[xyz][0]))
                #print ('\tmin')
                for tmp in range (8):  #kostil' dlya bezmorphinga
                    for xyz in range (3):
                        if tmp > 3:
                            fgr (pack ('f', act_m.min[xyz][0]*context.scene.scale))
                        else:
                            fgr (pack ('f', act_m.min[xyz][0]))
                #print ('\tmax')
                for tmp in range (8):  #kostil' dlya bezmorphinga
                    for xyz in range (3):
                        if tmp > 3:
                            fgr (pack ('f', act_m.max[xyz][0]*context.scene.scale))
                        else:
                            fgr (pack ('f', act_m.max[xyz][0]))
                #print ('\tradius')
                for tmp in range (8):  #kostil' dlya bezmorphinga
                    if tmp > 3:
                        fgr (pack ('f', act_m.radius[0]*context.scene.scale))
                    else:
                        fgr (pack ('f', act_m.radius[0]))
                #print ('\tverts')
                ib=0
                for v_c in range (act_m.header[0]):
                    for xyz in range (3):
                        for tmp1 in range (8):  #kostil' dlya bezmorphinga
                            for tmp2 in range (4):
                                if tmp1 > 3:
                                    fgr (pack ('f', act_m.m_verts[xyz][ib+tmp2]*context.scene.scale))
                                else:
                                    fgr (pack ('f', act_m.m_verts[xyz][ib+tmp2]))
                                
                    ib+=4
                #print ('\tnormals')
                nb=0
                for n_c in range (act_m.header[1]):
                    for xyzw in range (4):
                        for tmp in range (4):
                            fgr (pack ('f', act_m.normals[nb+tmp][xyzw]))
                    nb+=4
                #print ('\ttexture coordinates')
                for t_c in range (act_m.header[2]):
                    for tmp in range (2):
                        fgr (pack ('f', act_m.t_coords[t_c][tmp]))
                #print ('\tindicies')
                for ii in range (act_m.header[3]):
                    fgr (pack ('h', act_m.indicies[ii]))
                #print ('\tv_c')
                for v_c_i in range (act_m.header[4]):
                    for tmp in range (2):
                        fgr (pack ('h', act_m.v_c[v_c_i][0]))
                    fgr (pack ('h', act_m.v_c[v_c_i][1]))
                #print ('\tm_c')
                for m_c_i in range (act_m.header[5]):
                    for tmp in range (2):
                        fgr (pack ('h', m_c_i))
                del(act_m)
                fg.close()

        if not path_file.lower().endswith(".lnk"):
            path_file += ".lnk"
        scene = bpy.context.scene
        for ob in bpy.data.objects:
            bpy.ops.object.mode_set(mode='OBJECT')
            scene.objects.active = ob
            bpy.ops.object.transform_apply(location=True, rotation=True, scale=True)
        meshclean()
        export_lnk()
        export_bon()
            
        for obj in bpy.data.objects:
            if obj.type=='MESH':
                scene.objects.active = obj
                triangulate_object(obj)
        export_fig()

        self.report({'INFO'}, "finished export")
        return {'FINISHED'}
    
    def invoke(self, context, event):
        WindowManager = context.window_manager
        WindowManager.fileselect_add(self)
        return {"RUNNING_MODAL"}

class EImodelImport(bpy.types.Operator):
    bl_label = "EI model import Operator"
    bl_idname = "object.eimodelimport"
    bl_description = "Importing model file from Evil Islands"
    filepath = bpy.props.StringProperty(subtype='FILE_PATH')

    def execute(self, context):
        self.report({'INFO'}, "executing import")
        path_file = self.filepath
        count_mesh=0
        bonebase=''
        for mesh in bpy.data.meshes:
            count_mesh+=1
        rou=4
        class cei_mesh:
            pass

        act_m=cei_mesh()

        if '.lnk' in path_file:
            f = open (path_file, 'rb')
            bones=dict()
            mname=''
            s=unpack('i', f.read(4))
            s=s[0]
            for l in range (s):
                tmp1=unpack('i', f.read(4))
                ftmp1=tmp1[0]
                mname=unpack(str(ftmp1-1)+'s', f.read(ftmp1-1))[0].decode()
                f.read(1)
                figfile = os.path.dirname(path_file)+'\\'+mname+'.fig'
                bonfile = os.path.dirname(path_file)+'\\'+mname+'.bon'
                tmp3=unpack('i', f.read(4))
                ftmp3=tmp3[0]
                if ftmp3==0:
                    pass
                else:
                    bonebase=unpack(str(ftmp3-1)+'s', f.read(ftmp3-1))[0].decode()
                    f.read(1)
                try:
                    ff=open (figfile, 'rb')
                    act_m.header=[0 for i in range (9)]
                    act_m.center=[[], [], []]
                    act_m.m_verts=[[], [], []]
                    act_m.normals=[[], [], [], []]
                    act_m.t_coords=[[], []]
                    act_m.indices=[]
                    act_m.v_c=[[], [], []]
                    act_m.bon=[[], [], []]
                    while 1==1:
                        #SIGNATURE
                        ff.read(4)
                        #HEADER
                        for c in range (9):
                            tmp=unpack('i', ff.read(4))
                            act_m.header[c]=tmp[0]
                        context.scene.group=act_m.header[7]
                        context.scene.t_number=act_m.header[8]
                        
                        ff.read(320)
                        
                        #VERTICES
                        ccc = act_m.header[0]
                        for b24 in range (ccc):
                            for i1 in range (3):
                                for i2 in range (4):
                                    tmp=unpack('f', ff.read(4))
                                    act_m.m_verts[i1].append(tmp[0])
                                ff.read (112)
                        
                        #NORMALS
                        ccn = act_m.header[1]
                        for b4 in range (ccn):
                            for n1 in range (4):
                                for n2 in range (4):
                                    tmp=unpack('f', ff.read(4))
                                    act_m.normals[n1].append(tmp[0])

                        #TEXTURE COORDS
                        tc = act_m.header[2]
                        for t_c in range (tc):
                            tmpx=unpack('f', ff.read(4))
                            tmpy=unpack('f', ff.read(4))
                            act_m.t_coords[0].append(tmpx[0])
                            act_m.t_coords[1].append(tmpy[0])

                        #INDICES
                        ic=act_m.header[3]
                        for i_c in range (ic):
                            tmp=unpack('h', ff.read(2))
                            act_m.indices.append(tmp[0])

                        #VERTICES COMPONENTS
                        vc = act_m.header[4]
                        for v_c in range (vc):
                            for v_c_i in range (3):
                                tmp=unpack('h', ff.read(2))
                                act_m.v_c[v_c_i].append(tmp[0])
                        break
                    
                    ff.close()
                    #BON-FILES
                    try:
                        fb=open (bonfile, 'rb')
                        base=os.path.basename(bonfile)
                        bname=os.path.splitext(base)[0]
                        bones[bname]=[0.0, 0.0, 0.0]
                        val=bones[bname]
                        for orig in range (3):
                            btmp=unpack('f', fb.read(4))
                            act_m.bon[orig].append(btmp[0])
                            val[orig]=btmp[0]
                        fb.close()
                    except FileNotFoundError:
                        base=os.path.basename(bonfile)
                        bname=os.path.splitext(base)[0]
                        bones[bname]=[0.0, 0.0, 0.0]
                        for orig in range (3):
                            act_m.bon[orig].append(0.0)
                    #peres4et novih kostei
                    if bonebase != '':
                        old_base=os.path.basename(bonfile)
                        old_name=os.path.splitext(old_base)[0]
                        old_val=bones[old_name]
                        new_base=os.path.basename(bonebase)
                        new_name=os.path.splitext(new_base)[0]
                        new_val=bones[new_name]
                        
                        old_val[0]=old_val[0]+copy.copy(new_val[0])
                        old_val[1]=old_val[1]+copy.copy(new_val[1])
                        old_val[2]=old_val[2]+copy.copy(new_val[2])
                        
                    #VERTICES
                    verts=[]
                    vtemp=[0.0, 0.0, 0.0]
                    vi=act_m.header[0]*4
                    v_count=0
                    for v in range (vi):
                        for xyz in range (3):
                            vtemp[xyz]=copy.copy(act_m.m_verts[xyz][v])
                        verts.append([vtemp[0], vtemp[1], vtemp[2]])
                        v_count+=1

                    #FACES
                    faces=[]
                    ftemp=[0, 0, 0]
                    fi=act_m.header[3]-2
                    f_count=0
                    fc=0
                    while fc < fi:
                        for vvv in range (3):
                            ftemp[vvv]=act_m.v_c[0][act_m.indices[fc+vvv]]
                        faces.append([ftemp[0], ftemp[1], ftemp[2]])
                        f_count+=1
                        fc+=3

                    #=====>CREATING MESH<=====
                    me=bpy.data.meshes.new(name=mname)
                    obj=bpy.data.objects.new(mname, me)
                    scn=bpy.context.scene
                    value_bon=bones[mname]
                    obj.location=(value_bon[0], value_bon[1], value_bon[2])
                    scn.objects.link (obj)
                    scn.objects.active=obj
                    obj.select=True
                    me.from_pydata(verts, [], faces)

                    #UV COORDINATES !
                    fffname=os.path.basename(path_file)
                    fm_name=os.path.splitext(fffname)[0]
                    me.uv_textures.new(name=fm_name)
                    ti = copy.copy(act_m.header[3])
                    for t in range (ti):
                        me.uv_layers[0].data[t].uv[0]=act_m.t_coords[0][act_m.v_c[2][act_m.indices[t]]]
                        me.uv_layers[0].data[t].uv[1]=act_m.t_coords[1][act_m.v_c[2][act_m.indices[t]]]
                    me.update()

                except FileNotFoundError:
                    print ('fig-file not found: '+figfile)
            f.close()
        #end of lnk-branch

        if '.fig' in path_file:
            f1 = open (path_file, 'rb')
            act_m.header=[0 for i in range (9)]
            act_m.center=[[], [], []]
            act_m.m_verts=[[], [], []]
            act_m.normals=[[], [], [], []]
            act_m.t_coords=[[], []]
            act_m.indices=[]
            act_m.v_c=[[], [], []]
            act_m.bon=[[], [], []]
            while 1==1:
                #SIGNATURE
                f1.read(4)
                #HEADER
                for c in range (9):
                    tmp=unpack('i', f1.read(4))
                    act_m.header[c]=tmp[0]
                context.scene.group=act_m.header[7]
                context.scene.t_number=act_m.header[8]

                f1.read(320)
                
                #VERTICES
                m_verts_count=0
                ccc = act_m.header[0]
                for b24 in range (ccc):
                    for i1 in range (3):
                        for i2 in range (4):
                            tmp=unpack('f', f1.read(4))
                            act_m.m_verts[i1].append(tmp[0])
                            m_verts_count+=1
                        f1.read (112)

                #NORMALS
                ccn = act_m.header[1]
                for b4 in range (ccn):
                    for n1 in range (4):
                        for n2 in range (4):
                            tmp=unpack('f', f1.read(4))
                            act_m.normals[n1].append(tmp[0])

                #TEXTURE COORDS
                tc = act_m.header[2]
                for t_c in range (tc):
                    tmpx=unpack('f', f1.read(4))
                    tmpy=unpack('f', f1.read(4))
                    act_m.t_coords[0].append(tmpx[0])
                    act_m.t_coords[1].append(tmpy[0])

                #INDICES
                ic=act_m.header[3]
                for i_c in range (ic):
                    tmp=unpack('h', f1.read(2))
                    act_m.indices.append(tmp[0])
                
                #VERTICES COMPONENTS
                vc = act_m.header[4]
                for v_c in range (vc):
                    for v_c_i in range (3):
                        tmp=unpack('h', f1.read(2))
                        act_m.v_c[v_c_i].append(tmp[0])

                #BON FILE
                try:
                    fb=open (os.path.splitext(path_file)[0]+'.bon', 'rb')
                    for orig in range (3):
                        btmp=unpack('f', fb.read(4))
                        act_m.bon[orig].append(btmp[0])
                    fb.close()
                except FileNotFoundError:
                    for orig in range (3):
                        act_m.bon[orig].append(0.0)

                f1.close()
                break

            #===>>>>>> CREATING MESH <<<<<====
            #VERTICES
            verts=[]
            vtemp=[0.0, 0.0, 0.0]
            vi=act_m.header[0]*4
            v_count=0
            for v in range (vi):
                for xyz in range (3):
                    vtemp[xyz]=copy.copy(act_m.m_verts[xyz][v])
                verts.append([vtemp[0], vtemp[1], vtemp[2]])
                v_count+=1

            #FACES
            faces=[]
            ftemp=[0, 0, 0]
            fi=act_m.header[3]-2
            f_count=0
            f=0
            while f < fi:
                for vvv in range (3):
                    ftemp[vvv]=act_m.v_c[0][act_m.indices[f+vvv]]
                faces.append([ftemp[0], ftemp[1], ftemp[2]])
                f_count+=1
                f+=3

            #=====>MESH IN SCENE<======
            fname=os.path.basename(path_file)
            m_name=os.path.splitext(fname)[0]
            me=bpy.data.meshes.new(name=m_name)
            obj=bpy.data.objects.new(m_name, me)
            scn=bpy.context.scene
            obj.location=(act_m.bon[0][0], act_m.bon[1][0], act_m.bon[2][0])
            scn.objects.link (obj)
            scn.objects.active=obj
            obj.select=True
            me.from_pydata(verts, [], faces)

            #UV COORDINATES
            me.uv_textures.new(name=me.name)
            ti = copy.copy(act_m.header[3])
            for t in range (ti):
                me.uv_layers[0].data[t].uv[0]=act_m.t_coords[0][act_m.v_c[2][act_m.indices[t]]]
                me.uv_layers[0].data[t].uv[1]=act_m.t_coords[1][act_m.v_c[2][act_m.indices[t]]]
            me.update()
        self.report({'INFO'}, "finished import")
        return {'FINISHED'}
    
    def invoke(self, context, event):
        WindowManager = context.window_manager
        WindowManager.fileselect_add(self)
        return {"RUNNING_MODAL"}
 
def register():
    bpy.utils.register_module(__name__)
    bpy.types.Scene.t_number = bpy.props.IntProperty(name = "texture_number", default=0, min=0, max=20)
    bpy.types.Scene.group = bpy.props.IntProperty(name = "group", default=0, min=0, max=30)
    bpy.types.Scene.scale = bpy.props.FloatProperty(name = "scale", default=1, min=0, max=3)
 
 
def unregister():
    bpy.utils.unregister_module(__name__)
    bpy.types.Scene.t_number
    bpy.types.Scene.group
    bpy.types.Scene.scale
 
if __name__ == "__main__":
    register()